"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.InterceptionProxyRequest = void 0;
const events_1 = __importDefault(require("events"));
const puppeteer_tough_cookie_store_1 = require("puppeteer-tough-cookie-store");
const tough_cookie_1 = require("tough-cookie");
const interfaces_1 = require("../interfaces");
const mixins_1 = require("../mixins");
const Response_1 = require("./Response");
const cors_1 = require("../utils/cors");
const errors_1 = require("../utils/errors");
const getCDPSession_1 = require("../utils/getCDPSession");
const _OriginalRequestStateManager_1 = require("./_OriginalRequestStateManager");
let RequestBase = class RequestBase extends events_1.default {
};
RequestBase = __decorate([
    mixins_1.applyConfigurableMixin,
    mixins_1.applyLoggableMixin,
    mixins_1.applyNetworkMixin
], RequestBase);
for (let key of ['method', 'url', 'headers', 'body']) {
    Object.defineProperty(RequestBase.prototype, key, {
        get: function () {
            return this.requestOptions[key];
        },
        set: function (value) {
            return this.__requestOptionSetter(key, value);
        },
        enumerable: true,
        configurable: true
    });
}
/**
 * Plugin' request. The request have significant difference with Puppeteer' request.
 */
class InterceptionProxyRequest extends RequestBase {
    constructor(initial, requestOptions) {
        super();
        this.__isAdjusted = false;
        this.page = initial.page;
        this.originalRequest = initial.originalRequest;
        this.requestOptions = requestOptions;
        this.__stage = interfaces_1.RequestStage.gotRequest;
        this.__parent = initial.__parent;
    }
    emit2(eventName, payload) {
        var _a;
        this.emit(eventName, payload);
        (_a = this.__response) === null || _a === void 0 ? void 0 : _a.emit(eventName, payload);
    }
    static async proceedNewRequest(initial) {
        const { page, originalRequest, __parent } = initial;
        const originalRequestStateManager = new _OriginalRequestStateManager_1._OriginalRequestStateManager(page, originalRequest);
        const requestOptions = {
            method: originalRequest.method(),
            url: originalRequest.url(),
            headers: Object.assign({}, originalRequest.headers()),
            body: originalRequest.postData(),
        };
        (0, cors_1.adjustRequestCorsHeaders)(initial, requestOptions);
        const isRequestClientClosed = () => {
            const client = (0, getCDPSession_1.getCDPSession)(page, originalRequest);
            return !client.connection();
        };
        try {
            if (__parent.enableLegacyCookieHandling) {
                const cookieJar = new tough_cookie_1.CookieJar(new puppeteer_tough_cookie_store_1.PuppeteerToughCookieStore((0, getCDPSession_1.getCDPSession)(__parent.page, originalRequest)));
                requestOptions.headers['Cookie'] = await cookieJar.getCookieString(requestOptions.url);
            }
        }
        catch (error) {
            if (isRequestClientClosed())
                return;
            __parent.logger({
                level: 'warning',
                error: error,
                message: `Getting cookie error but we will proceed without\n` +
                    (error instanceof Error ? error.message : '' + error),
                meta: [requestOptions.url],
            });
        }
        ;
        const request = new InterceptionProxyRequest(initial, requestOptions);
        // native response listener handlers
        request.once('responseInstance', () => {
            originalRequestStateManager.close();
        });
        originalRequestStateManager.onResponse(async (originalResponse) => {
            if (request.__response)
                return;
            request.stage = interfaces_1.RequestStage.sentRequest;
            const response = originalResponse ?
                await Response_1.InterceptionProxyResponse.proceedOriginalResponse(request, originalResponse) :
                new Response_1.InterceptionProxyResponse(request, { abortReason: 'failed' });
            request._setResponseInstance(response, interfaces_1.RequestMode.native);
        });
        for (let handlerObj of request.requestListeners) {
            try {
                const localRes = await handlerObj.handler(request);
                if (localRes)
                    break;
            }
            catch (error) {
                if (isRequestClientClosed())
                    return;
                request.recordError(`Your interception handler "${String(handlerObj.key)}" throws an error. ` +
                    `Please wrap your function using try-catch blocks. For now we will ignore an error`, error);
            }
            if (isRequestClientClosed())
                return;
        }
        ;
        try {
            if (request.isResponseOverrideAvailable &&
                !('ignoreResponseBodyIfPossible' in request.getLocalConfiguration())) {
                request.ignoreResponseBodyIfPossible = request.ignoreResponseBodyIfPossible;
            }
            await request.continue();
        }
        catch (error) {
            if (isRequestClientClosed())
                return;
            request.recordError('Unable to proceed response from interception handler. We will abort the request.', error);
            try {
                await originalRequest.abort('failed', request.cooperativePriority);
            }
            catch (error) {
                // we already said that something went wrong
            }
        }
    }
    get stage() { return this.__stage; }
    ;
    set stage(value) {
        this.__stage = value;
        this.emit2('stage', value);
    }
    ;
    __requestOptionSetter(key, value) {
        if (!this.isRequestOverrideAvailable) {
            throw new Error((0, errors_1.getStageEnhancedErrorMessage)(key, this.stage));
        }
        if (this.requestMode !== interfaces_1.RequestMode.managed) {
            this.recordWarning(`Possibly the request will not be upgraded by your "${key}" ` +
                `because current mode ${this.requestMode} does not support this ability. ` +
                `Please set "requestMode" variable to "${interfaces_1.RequestMode.managed}" and then try again.`);
        }
        if (!this.__isAdjusted)
            this.__isAdjusted = true;
        this.requestOptions[key] = value;
        this.emit2(key, value);
        this.emit2('requestOptions', this.requestOptions);
    }
    async _getResponseInstance(responseOptions) {
        const { requestMode } = this;
        // @ts-ignore: we are not much care about ignoring requests
        if (requestMode === interfaces_1.RequestMode.ignore)
            return null;
        if (this.isRequestOverrideAvailable && !responseOptions &&
            (this.requestMode === interfaces_1.RequestMode.native ||
                (this.requestMode === interfaces_1.RequestMode.managed && !this.__isAdjusted && this.nativeContinueIfPossible))) {
            this.stage = interfaces_1.RequestStage.sentRequest;
            this.originalRequest.continue(undefined, this.cooperativePriority);
        }
        if (this.__response || this.isResponseCollecting ||
            (requestMode === interfaces_1.RequestMode.native && !responseOptions)) {
            const response = this.__response || await new Promise(c => this.once('responseInstance', c));
            if (responseOptions) {
                response.setResponse(responseOptions);
            }
            return response;
        }
        this.stage = interfaces_1.RequestStage.sentRequest;
        const response = await Response_1.InterceptionProxyResponse.proceedNewResponse(this, responseOptions);
        this._setResponseInstance(response);
        return response;
    }
    ;
    // If its managed response we can modify data until it will be not sent to puppeteer,
    // but in another case it will be finale version of response
    _setResponseInstance(response, mode = interfaces_1.RequestMode.managed) {
        this.__response = response;
        this.stage = mode === interfaces_1.RequestMode.managed ? interfaces_1.RequestStage.gotResponse : interfaces_1.RequestStage.sentResponse;
        this.emit2('responseInstance', response);
    }
    getRequest() {
        return this;
    }
    ;
    async setResponse(response) {
        return await this._getResponseInstance(response);
    }
    async getResponse({ requestMode } = {}) {
        if (requestMode)
            this.requestMode = requestMode;
        return await this._getResponseInstance();
    }
    async abort(abortReason) {
        return await this._getResponseInstance({ abortReason: abortReason || 'aborted' });
    }
    async continue() {
        const response = await this.getResponse();
        await response.continue();
    }
}
exports.InterceptionProxyRequest = InterceptionProxyRequest;
//# sourceMappingURL=Request.js.map