"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.InterceptionProxyResponse = void 0;
const events_1 = __importDefault(require("events"));
const puppeteer_tough_cookie_store_1 = require("puppeteer-tough-cookie-store");
const tough_cookie_1 = require("tough-cookie");
const interfaces_1 = require("../interfaces");
const mixins_1 = require("../mixins");
const errors_1 = require("../utils/errors");
const getCDPSession_1 = require("../utils/getCDPSession");
let ResponseBase = class ResponseBase extends events_1.default {
};
ResponseBase = __decorate([
    mixins_1.applyNetworkMixin,
    mixins_1.applyLoggableMixin,
    mixins_1.applyConfigurableMixin
], ResponseBase);
for (let key of ['status', 'headers', /*'contentType',*/ 'body', 'abortReason']) {
    Object.defineProperty(ResponseBase.prototype, key, {
        get: function () {
            if (key === 'body' && this.responseOptions._bodyError)
                throw new Error(this.responseOptions._bodyError);
            return this.responseOptions[key];
        },
        set: function (value) {
            return this.__responseOptionSetter(key, value);
        },
        enumerable: true,
        configurable: true
    });
}
class InterceptionProxyResponse extends ResponseBase {
    constructor(__parent, responseOptions) {
        super();
        this.originalResponse = undefined;
        this.__parent = __parent;
        this.responseOptions = responseOptions;
        this.emit2('responseOptions', this.responseOptions);
    }
    emit2(eventName, payload) {
        this.emit(eventName, payload);
        this.__parent.emit(eventName, payload);
    }
    static async __getResponse(request) {
        for (let handlerObj of request.requestHandlers) {
            try {
                const responseOptions = await handlerObj.handler(request);
                if (responseOptions)
                    return responseOptions;
            }
            catch (error) {
                request.recordError(`Interception handler "${String(handlerObj.key)}" throws an error. ` +
                    `For now we will ignore an error`, error);
            }
        }
        ;
        throw new Error(`No requestHandlers left. ` +
            `Looks like ${interfaces_1.NPM_PACKAGE_NAME} doest not support urls like that` +
            `Please provide more information ${interfaces_1.AUTHOR_NEW_ISSUE_URL}`);
    }
    static async proceedNewResponse(request, responseOptions) {
        try {
            responseOptions !== null && responseOptions !== void 0 ? responseOptions : (responseOptions = await this.__getResponse(request));
        }
        catch (error) {
            responseOptions = { abortReason: 'connectionfailed' };
            request.recordError(`Unable to proceed url`, error, request.url);
        }
        const response = new InterceptionProxyResponse(request, responseOptions);
        return response;
    }
    static async proceedOriginalResponse(request, originalResponse) {
        const responseOptions = {
            status: originalResponse.status(),
            headers: originalResponse.headers(),
            body: '',
        };
        try {
            if (!request.getLocalConfiguration().ignoreResponseBodyIfPossible) {
                responseOptions.body = await originalResponse.buffer();
            }
            else {
                responseOptions._bodyError = `${interfaces_1.NPM_PACKAGE_NAME}: Response body was not resolved for performance. ` +
                    `Use 'ignoreResponseBodyIfPossible' option or 'await response.originalResponse.buffer()' to avoid this error`;
            }
        }
        catch (e) {
            responseOptions._bodyError = `${interfaces_1.NPM_PACKAGE_NAME}: Unable to get original response`;
            // console.dir(e);
            // Could not load body for this request. This might happen if the request is a pre a preflight request.
        }
        const response = new InterceptionProxyResponse(request, responseOptions);
        response.originalResponse = originalResponse;
        return response;
    }
    getRequest() { return this.__parent; }
    get page() { return this.__parent.page; }
    get originalRequest() { return this.__parent.originalRequest; }
    get stage() { return this.__parent.stage; }
    set stage(value) { this.__parent.stage = value; }
    __responseOptionSetter(key, value) {
        if (!this.isResponseOverrideAvailable) {
            throw new Error((0, errors_1.getStageEnhancedErrorMessage)(key, this.stage));
        }
        if (key === 'abortReason') {
            this.responseOptions = { abortReason: value };
        }
        else {
            if ("abortReason" in this.responseOptions)
                delete this.responseOptions.abortReason;
            // const responseOptions = this.responseOptions as IResponseOverrides
            // @ts-ignore
            this.responseOptions[key] = value;
            // if (!responseOptions.status) responseOptions.status = 200;
            // if (!responseOptions.headers) responseOptions.headers = {};
            // if (!responseOptions.body) responseOptions.body = '';
        }
        this.emit2(key, value);
        this.emit2('responseOptions', this.responseOptions);
    }
    async setResponse(responseOptions) {
        this.responseOptions = responseOptions;
        return this;
    }
    async getResponse() {
        return this;
    }
    async abort(abortReason) {
        this.responseOptions = { abortReason };
        return this;
    }
    async continue() {
        // prevent processing for already handled requests
        if (!this.isResponseOverrideAvailable) {
            if (this.requestMode !== interfaces_1.RequestMode.managed)
                return;
            // TODO: warn?
            return;
        }
        this.stage = interfaces_1.RequestStage.sentResponse;
        const response = this.responseOptions;
        try {
            if ("body" in response) {
                const cookieHeaderRaw = response.headers["set-cookie"];
                const cookieHeader = typeof cookieHeaderRaw === 'string' ? [cookieHeaderRaw] : cookieHeaderRaw;
                if (this.enableLegacyCookieHandling && cookieHeader) {
                    response.headers["set-cookie"] = undefined;
                    const cookieJar = new tough_cookie_1.CookieJar(new puppeteer_tough_cookie_store_1.PuppeteerToughCookieStore((0, getCDPSession_1.getCDPSession)(this.page, this.originalRequest)));
                    await Promise.all(cookieHeader.map(cookieString => {
                        const cookie = tough_cookie_1.Cookie.parse(cookieString);
                        if (!cookie)
                            return;
                        return cookieJar.setCookie(cookie, this.getRequest().url);
                    }));
                }
                await this.originalRequest.respond(response, this.cooperativePriority);
            }
            else if ("continue" in response) {
                await this.originalRequest.continue(undefined, this.cooperativePriority);
            }
            else {
                await this.originalRequest.abort(response.abortReason, this.cooperativePriority);
            }
        }
        catch (error) {
            this.stage = interfaces_1.RequestStage.closed;
            const client = (0, getCDPSession_1.getCDPSession)(this.page, this.originalRequest);
            if (client && !client.connection()) {
                // possibly page was closed, no need to scream
            }
            else {
                this.emit2('error', error);
            }
        }
    }
}
exports.InterceptionProxyResponse = InterceptionProxyResponse;
//# sourceMappingURL=Response.js.map