"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.baseHttpHandler = exports.baseHttpHandlerKey = void 0;
const got_1 = __importDefault(require("got"));
exports.baseHttpHandlerKey = 'baseHttpHandlerKey';
async function baseHttpHandler(request) {
    try {
        const { method, url, headers, body, timeout, agent, gotHooks } = request;
        if (!url.startsWith('http:') && !url.startsWith('https:'))
            return;
        const gotOptions = {
            // main request options
            method, url, headers, body, hooks: gotHooks,
            // secondary request options
            timeout,
            // another `got` options
            responseType: 'buffer',
            throwHttpErrors: false,
            ignoreInvalidCookies: true,
            followRedirect: false,
        };
        if (agent) {
            gotOptions.agent = {
                http: agent,
                https: agent,
            };
        }
        const gotResponse = await (0, got_1.default)(gotOptions);
        return {
            status: gotResponse.statusCode,
            headers: gotResponse.headers,
            // contentType: gotResponse.headers['content-type'],
            body: gotResponse.body,
        };
    }
    catch (error) {
        request.recordWarning('Unable to get response for', [request.url, error]);
        // TODO: throw correct `abortReason`
        return {
            abortReason: 'connectionreset'
        };
    }
}
exports.baseHttpHandler = baseHttpHandler;
//# sourceMappingURL=baseHttpHandler.js.map