"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RequestStage = exports.RequestMode = void 0;
/**
 * Plugin mode for handling requests
 */
var RequestMode;
(function (RequestMode) {
    /**
     * Plugin will do nothing about original request
     */
    RequestMode["ignore"] = "ignore";
    /**
     * Plugin will just listen to the original request/response data and all requests will fulfilled by puppeteer itself.
     * But some plugin functionality can be unavailable.
     */
    RequestMode["native"] = "native";
    /**
     * Plugin will do all requests by himself. All plugin features will be available.
     */
    RequestMode["managed"] = "managed";
})(RequestMode = exports.RequestMode || (exports.RequestMode = {}));
/**
 * Current stage of the request
 */
var RequestStage;
(function (RequestStage) {
    // * We just got a new request from the puppeteer and getting additional information about it.
    // initial = 'initial',
    /**
     * We got a new request from the puppeteer witch includes all necessary information about.
     *
     * At this stage we can adjust request.
     */
    RequestStage["gotRequest"] = "gotRequest";
    /**
     * The request in requesting process
     *
     * At this stage we unable to adjust request but still have not response to go forward.
     */
    RequestStage["sentRequest"] = "sentRequest";
    /**
     * We got response from the request witch probably was modified by the user and now user can adjust the response.
     *
     * At this stage we can adjust response.
     * At this stage the user will unable to override the request anymore.
     */
    RequestStage["gotResponse"] = "gotResponse";
    /**
     * We sent final response of the request to the browser.
     *
     * Its too late to adjust request or response.
     */
    RequestStage["sentResponse"] = "sentResponse";
    /**
     * Page were closed and we unable do anything
     *
     * From technical perspective `sentResponse` looks just the same
     */
    RequestStage["closed"] = "closed";
})(RequestStage = exports.RequestStage || (exports.RequestStage = {}));
//# sourceMappingURL=network.js.map