"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.applyNetworkMixin = void 0;
const interfaces_1 = require("../interfaces");
function applyNetworkMixin(base) {
    /**
     * Expecting this.stage in runtime
     */
    class NetworkMixin extends base {
        get isRequestOverrideAvailable() {
            return [
                // RequestStage.initial,
                interfaces_1.RequestStage.gotRequest,
            ].includes(this.stage);
        }
        get isResponseCollecting() {
            return this.stage === interfaces_1.RequestStage.sentRequest;
        }
        get isResponseCollected() {
            return [
                interfaces_1.RequestStage.gotResponse,
                interfaces_1.RequestStage.sentResponse,
            ].includes(this.stage);
        }
        get isResponseOverrideAvailable() {
            return [
                interfaces_1.RequestStage.gotRequest,
                interfaces_1.RequestStage.sentRequest,
                interfaces_1.RequestStage.gotResponse,
            ].includes(this.stage);
        }
        get isResponseFinished() {
            return [
                interfaces_1.RequestStage.sentResponse,
                interfaces_1.RequestStage.closed,
            ].includes(this.stage);
        }
        // todo: rework relations
        get json() {
            try {
                return JSON.parse(this.body);
            }
            catch (e) {
                return undefined;
            }
        }
        set json(obj) {
            this.body = JSON.stringify(obj);
        }
    }
    return NetworkMixin;
}
exports.applyNetworkMixin = applyNetworkMixin;
//# sourceMappingURL=NetworkMixin.js.map